;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;This script used to plot e3sm diagnostics for TCs;;;;;;
;TC Basin definitions.(Knutson et. al.,2020,BAMS)
;North Atlantic: 0-90N, 265-360E
;Northeast Pacific: 0-90N, 180-265E
;Northwest Pacific: 0-90N, 100-180E
;North Indian: 0-90N, 30-100E
;South Indian: 90S-0, 20-135E
;Southwest Pacific: 90S-0, 135-295E
;South Atlantic(SA1+SA2): 90S-0, South America (295-360E) to Africa (0-20E)
;The North Atlantic-northeast Pacific boundary is on a
;diagonal tracing a path through Mexico and Central America.
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
load "$NCARG_ROOT/lib/ncarg/nclscripts/csm/gsn_code.ncl"
load "$NCARG_ROOT/lib/ncarg/nclscripts/csm/gsn_csm.ncl"
load "$NCARG_ROOT/lib/ncarg/nclscripts/csm/contributed.ncl"
load "$NCARG_ROOT/lib/ncarg/nclscripts/esmf/ESMF_regridding.ncl"

begin

  ;;;;;;;;;;select the regions to calculate the rmse;;;;;;;;;;;;;
  datdir  = "./fig_data/"
  Groups  = (/"ERA5", "CLIM30", "CLIM30ML" /)
  LabList = (/"ERA5", "CLIM - ERA5", "ML(CLIM) - ERA5" /)
  ngrps   = dimsizes(Groups)

  Figstr  = (/"(d)", "(e)", "(f)"/)
  ystr    = 1979
  yend    = 2014
  nyr     = yend - ystr +1
  figkey  = "etc_composite"

  Varnam  = (/"TUQ", "TVQ","IVT"/)
  nvars   = dimsizes(Varnam)
  Varplt  = "ivt"
  Varstr  = "IVT"
  Varunt  = "kg m~S~-1~N~ s~S~-1~N~"

  do i = 0,ngrps-1,1

   do iv = 0,nvars-1,1
     fna = datdir+ Groups(i)+ "_etc_composite.nc"
     vnm = Varnam(iv)
     fla = addfiles(fna, "r")
     ListSetType (fla, "join")

     lat = fla[0]->lat
     lon = fla[0]->lon
     if(iv.eq.0) then 
       vdat = new((/3,nyr,dimsizes(lat),dimsizes(lon)/),typeof(lat))
       vdat!0 = "var"
       vdat!1 = "time"
       vdat!2 = "lat"
       vdat!3 = "lon"
       vdat&time = ispan(ystr,yend,1)
       vdat&lat  = lat 
       vdat&lon  = lon 
     end if 

     vdat(iv,:,:,:) = (/fla[:]->$vnm$/)

     if (i.eq.0) then 
      if(vnm.eq."IVT") then
       printMinMax(vdat(iv,:,:,:),False)
        lmsk = conform(vdat,vdat(iv,:,:,:),(/1,2,3/))
        lmsk = where(lmsk.lt.100,0.0,1.0) 
      end if
     end if 
     delete([/fna,vnm,fla,lat,lon/])
   end do 

   if(i.eq.0) then 
     vref = vdat
     plot_ttest = False
   else
     xAve = dim_avg_n_Wrap(vref(2,:,:,:),0)
     yAve = dim_avg_n_Wrap(vdat(2,:,:,:),0)
     xVar = dim_variance_n(vref(2,:,:,:),0)
     yVar = dim_variance_n(vdat(2,:,:,:),0)
     printMinMax(xAve,False)
     printMinMax(yAve,False)
     printMinMax(xVar,False)
     printMinMax(yVar,False)
     printVarSummary(vref(2,:,:,:))
     printVarSummary(vdat(2,:,:,:))
     printMinMax(vdat,False)
     printMinMax(vref,False)

     xN   = nyr
     yN   = nyr
     sigr = 0.05                                   ; critical sig lvl for r
     iflag = False                        ; population variance similar
     prob  = yAve
     prob  = (/ttest(xAve,xVar,xN, yAve,yVar,yN, iflag, False)/)
     prob  = where(prob.lt.sigr,prob,prob@_FillValue )
     plot_ttest  = True
     vdat = vdat - vref
     printMinMax(prob,False)
     delete([/xAve,yAve,xVar,yVar,sigr,xN,yN,iflag/])
   end if 
   vdat = where(lmsk.eq.0.0,vdat@_FillValue,vdat)
   varm = dim_avg_n_Wrap(vdat,1) 

   ;----------------------------------------------------------------------
   ; Plotting section
   ;----------------------------------------------------------------------

   if(i.eq.0)then
    ;;;;;;;open a wks for plot;;;;;;;;;;;;;;;;;;;
    wtype = "pdf"
    wtype@wkPaperSize  = "A4"
   ;wtype@wkOrientation   = "landscape" ;;pictures displayed along the length
 
    plotname = Varplt + "_" + figkey 
    wks1     = gsn_open_wks(wtype,plotname)
    plot     = new (ngrps, graphic)

    print(" ")
    print("........................... ")
    print("plot : " + plotname+".pdf")
    print("........................... ")

   end if 

   ;;;set up the plot information;;;;
   lndcolor   = "Black" ;"Grey50"
   cnlevels  := fspan(50,200,11) ;fspan(960,1008,13) 
   cnlevel2  := (/-50,-40,-30,-20,-5,5,20,30,40,50/)

;---Resources to share between both plots
   res                = True     ; Plot modes desired.
   res@gsnDraw        = False
   res@gsnFrame       = False
   res@gsnMaximize    = False     ; Maximize plot
   res@tmXTOn         = False
   res@tmYROn         = False

   BorderThick = 1.0
   res@tmBorderThicknessF    = BorderThick
   res@tmXBMajorThicknessF   = BorderThick
   res@tmXBMinorThicknessF   = BorderThick*0.5
   res@tmYLMajorThicknessF   = BorderThick
   res@tmYLMinorThicknessF   = BorderThick*0.5
   res@tmXTMajorThicknessF   = BorderThick
   res@tmXTMinorThicknessF   = BorderThick*0.5
   res@tmYRMajorThicknessF   = BorderThick
   res@tmYRMinorThicknessF   = BorderThick*0.5

   FontHeightF = 0.030
   res@tiMainFontHeightF    = FontHeightF
   res@tmYLLabelFontHeightF = FontHeightF
   res@tmXBLabelFontHeightF = FontHeightF

   mpres  = res
   resx   = res
   resx1  = res

   res@gsnStringFontHeightF  = FontHeightF*1.0
   res@lbLabelFontHeightF    = FontHeightF*0.9
   res@lbTitleFontHeightF    = FontHeightF*0.8
   res@tiXAxisFontHeightF    = FontHeightF*1.0
   res@tiYAxisFontHeightF    = FontHeightF*1.0

   res@cnFillOn              = True     ; color plot desired
   res@cnLinesOn             = False    ; turn off contour lines
   res@cnLineLabelsOn        = False    ; turn off contour labels
  ;res@cnFillMode            = "RasterFill"   ; turn raster on
   res@cnMissingValFillColor = "White"

   res@lbLabelBarOn             = True   ; Will turn on in panel later
   res@pmLabelBarParallelPosF   = 0.50 ;61
   res@pmLabelBarOrthogonalPosF = 0.05
   res@lbOrientation            = "vertical"  ; vertical label bar
   res@pmLabelBarHeightF        = 0.60
   res@pmLabelBarWidthF         = 0.08
   res@lbTitlePosition          = "Right"         ; title location
   res@lbTitleDirection         = "Across"        ; letter angle
   res@lbTitleAngleF            = 90.          ; title angle
   res@lbBoxLinesOn             = True

   res@gsnCenterString    = ""
   res@gsnLeftString      = ""
   res@gsnRightString     = "" 
   res@tiMainString       = "" 

   res@trYMinF = -25
   res@trYMaxF =  25
   res@trXMinF = -25
   res@trXMaxF =  25
   res@trYReverse = True 

   res@tmXBMode          = "Explicit"              ; explicit labels
   res@tmXBValues        = ispan(-30,30,10)        ; location of labels 
   res@tmXBLabels        = ispan(-30,30,10)
   res@tmXBMinorValues   = ispan(-30,30,5)
   res@tiXAxisString     = "X (Degree)"       ; Put label on X axis

   res@tmYLMode          = "Explicit"              ; explicit labels
   res@tmYLValues        = ispan(-30,30,10)
   res@tmYLLabels        = ispan(-30,30,10)
   res@tmYLMinorValues   = ispan(-30,30,5)
   res@tiYAxisString     = "Y (Degree)"       ; Put label on X axis

   res@tmXMajorGrid                = False         ; implement x grid 
   res@tmXMajorGridThicknessF      = 1.0           ; 2.0 is default
   res@tmXMajorGridLineDashPattern = 2             ; select short dash lines
  
   res@tmYMajorGrid                = False         ; implement y grid
   res@tmYMajorGridThicknessF      = 1.0           ; 2.0 is default
   res@tmYMajorGridLineDashPattern = 2             ; select short dash lines

  ;---Resources for plotting original data
   res1=res
   res1@cnLevelSelectionMode        = "ExplicitLevels"  
   res1@cnSmoothingOn               = True
   res1@cnLinesOn                   = True ;plot_contour  
   res1@cnLineLabelsOn              = False  
   res1@cnInfoLabelOn               = False
   res1@cnLineLabelDensityF         = 2.0
   res1@cnLineLabelFontHeightF      = 0.020
   res1@cnLineLabelBackgroundColor  = -1
   res1@gsnContourLineThicknessesScale = 1.0
   res1@cnLineThicknessF            = 0.001
   res1@cnLineColor                 = "Black"
   res1@gsnContourZeroLineThicknessF= 0.2
   res1@gsnContourNegLineDashPattern= 0

   res1@gsnLeftString               = Figstr(i) + "  " +LabList(i)
   res1@gsnLeftStringOrthogonalPosF = 0.04
   res1@gsnRightString              = ""; Varstr 
   res1@gsnRightStringOrthogonalPosF= 0.04

   resV=True
   resV@gsnFrame                = False
   resV@gsnDraw                 = False
   resV@vcLineArrowColor        = "Grey25"
   resV@vcRefLengthF            = 0.040  ; reference length
   resV@vcGlyphStyle            = "LineArrow";"CurlyVector"  ; curly vectors
   resV@vcMinDistanceF          = 0.03  ; minimum distance
   resV@vcLineArrowThicknessF   = 1.5 ;arrow thickness
   resV@vcRefAnnoFontHeightF    = FontHeightF*0.8 ; label size
   resV@vcRefAnnoOn             = True ; turn label on
   resV@vcRefAnnoString2On      = False ; turn default string off
   resV@vcRefAnnoOrthogonalPosF = -0.24 ; label position
   resV@vcRefAnnoParallelPosF   = 0.99
   resV@vcRefAnnoPerimSpaceF    = 0.33 ; label box space
   resV@vcRefAnnoArrowSpaceF    = 0.7


   if (i.eq.0) then  
     res1@cnLevels              := cnlevels
     res1@cnFillPalette         := "MPL_rainbow" 
     refmag = 200.
     resV@vcMinDistanceF         = 0.025  ; minimum distance
     res1@lbTitleString          = "IVT (kg m~S~-1~N~ s~S~-1~N~)"
   else
     res1@cnLevels              := cnlevel2
     res1@cnFillPalette         := "CBR_coldhot" ;"hotcolr_19lev" 
     refmag = 50
     resV@vcMinDistanceF         = 0.025  ; minimum distance
     res1@lbTitleString          = "Bias in IVT (kg m~S~-1~N~ s~S~-1~N~) "
   end if
   resV@vcRefMagnitudeF          = refmag      ; reference magnitude
   resV@vcRefAnnoString1         = refmag +"m/s"
   plot(i) = gsn_csm_contour(wks1,varm(2,:,:),res1)
   plotu   = gsn_csm_vector(wks1,varm(0,:,:),varm(1,:,:),resV) ; arrows drawing
  ;overlay(plot(i),plotu) ; overlaying

   if(plot_ttest) then
     res21 = res
     res21@gsnLeftString                = ""
     res21@gsnRightString               = ""
     res21@gsnCenterString              = ""
     res21@cnMissingValFillColor        = "Transparent"
     res21@cnFillOn                     = True
     res21@cnLinesOn                    = False
     res21@cnLineLabelsOn               = False       ; turn off contour labels
     res21@cnInfoLabelOn                = False
     res21@cnMonoFillColor              = True        ; default color is fground [black]
     res21@cnMonoFillPattern            = True
     res21@cnMonoFillScale              = True
     res21@cnFillPattern                = 17
     res21@cnFillScaleF                 = 0.9
     res21@cnFillDotSizeF               = 0.001
     res21@lbLabelBarOn                 = False
     if(.not.isvar("dumx")) then 
       dumx = new (ngrps,graphic)
     end if 
     printMinMax(prob,False)
     dumx(i) = gsn_csm_contour(wks1,prob,res21)
     overlay(plot(i),dumx(i))
     delete([/res21,prob/])
   end if

   ;************************************************
   ; add error bars
   ;************************************************  
   polyres                   = True                       ; marker resources
   polyres@gsMarkerIndex     = 5                          ; index for circle
   polyres@gsMarkerSizeF     = .008                        ; size 
   polyres@gsMarkerThicknessF= 2.0
   polyres@gsMarkerColor     = "black"
   if(.not.isvar("dum")) then
     dum = new(ngrps,graphic)
   end if 
   dum(i) = gsn_add_polymarker(wks1,plot(i),(/0,0/),(/0,0/),polyres)  

   delete([/res,res1,mpres/])

 end do ; end of experiment group loop

;---Draw both plots in a panel
;***************************************
; panel first set of plots
;***************************************
  pres1                          = True
  pres1@gsnFrame                 = True      ; don't advance frame yet
  pres1@gsnMaximize              = False
  pres1@gsnPanelLabelBar         = False ;True
  pres1@pmLabelBarParallelPosF   = 0.0
  pres1@pmLabelBarOrthogonalPosF = 0.1
 ;pres1@lbOrientation            = "vertical"  ; vertical label bar
  pres1@pmLabelBarHeightF        = 0.05 ;0.21
  pres1@pmLabelBarWidthF         = 0.75 ;0.05
  pres1@lbLabelFontHeightF       = FontHeightF * 0.6
  pres1@lbTitleFontHeightF       = FontHeightF * 0.65
  pres1@lbTitleString            = "TC composite total precipitation (mm/day)"
  pres1@lbTitlePosition          = "Left";"Right"         ; title location
 ;pres1@lbTitleDirection         = "Across"        ; letter angle
 ;pres1@lbTitleAngleF            = 90.          ; title angle
  pres1@lbBoxLinesOn             = True
 ;pres1@lbBoxSeparatorLinesOn    = False
 ;pres1@lbLabelStride            = 15

  pres1@gsnPanelMainString       = " "
  pres1@gsnPanelMainFontColor    = "Black"
  pres1@gsnPanelMainFontHeightF  = 0.028
  pres1@gsnPanelYWhiteSpacePercent = 2
  pres1@gsnPanelXWhiteSpacePercent = 2
  gsn_panel(wks1,(/plot/),(/1,ngrps/),pres1)

end
